@@ -19,39 +19,44 @@ def _compare_table(input_table):
     """
     rows_expected = 4
     epoch_expected = [1234, 1235, 1236, 1237]
-    object_expected = [1,1,1,0]
+    object_expected = [1, 1, 1, 0]
     quant1_expected = [0.01, 1.0, 1.0, 10.0]
     quant1_err_expected = [0.005, 0.005, 0.005, 0.1]
     quant2_expected = [0.5, 89.0, 89.3, np.nan]
     quant2_err_expected = [0.05, 0.1, 0.3, np.nan]
     quant_type_expected = ['radec', 'seppa', 'seppa', 'rv']
+    instrument_expected = ['defrd', 'defsp', 'defsp', 'defrv']
     assert len(input_table) == rows_expected
-    for meas,truth in zip(input_table['epoch'],epoch_expected):
+    for meas, truth in zip(input_table['epoch'], epoch_expected):
         assert truth == pytest.approx(meas)
-    for meas,truth in zip(input_table['object'],object_expected):
+    for meas, truth in zip(input_table['object'], object_expected):
         assert truth == meas
-    for meas,truth in zip(input_table['quant1'],quant1_expected):
+    for meas, truth in zip(input_table['quant1'], quant1_expected):
         if np.isnan(truth):
             assert np.isnan(meas)
         else:
             assert truth == pytest.approx(meas)
-    for meas,truth in zip(input_table['quant1_err'],quant1_err_expected):
+    for meas, truth in zip(input_table['quant1_err'], quant1_err_expected):
         if np.isnan(truth):
             assert np.isnan(meas)
         else:
             assert truth == pytest.approx(meas)
-    for meas,truth in zip(input_table['quant2'],quant2_expected):
+    for meas, truth in zip(input_table['quant2'], quant2_expected):
         if np.isnan(truth):
             assert np.isnan(meas)
         else:
             assert truth == pytest.approx(meas)
-    for meas,truth in zip(input_table['quant2_err'],quant2_err_expected):
+    for meas, truth in zip(input_table['quant2_err'], quant2_err_expected):
         if np.isnan(truth):
             assert np.isnan(meas)
         else:
             assert truth == pytest.approx(meas)
-    for meas,truth in zip(input_table['quant_type'],quant_type_expected):
-            assert truth == meas
+    for meas, truth in zip(input_table['quant_type'], quant_type_expected):
+        assert truth == meas
+
+    for meas, truth in zip(input_table['instrument'], instrument_expected):
+        assert truth == meas
+
 
 def test_read_file():
     """
@@ -64,6 +69,7 @@ def test_read_file():
     input_file_radec = os.path.join(orbitize.DATADIR, 'test_val_radec.csv')
     read_file(input_file_radec)
 
+
 @deprecation.fail_if_not_removed
 def test_read_formatted_file():
     """
@@ -78,6 +84,7 @@ def test_read_formatted_file():
     input_file_radec = os.path.join(orbitize.DATADIR, 'test_val_radec.csv')
     read_file(input_file_radec)
 
+
 def test_write_orbitize_input():
     """
     Test the write_orbitize_input and the read_file functions
@@ -90,7 +97,7 @@ def test_write_orbitize_input():
         os.remove(output_file)
     try:  # Catch these tests so that we remove temporary file
         # Test that we were able to write the table
-        write_orbitize_input(test_table,output_file)
+        write_orbitize_input(test_table, output_file)
         assert os.path.isfile(output_file)
         # Test that we can read the table and check if it's correct
         test_table_2 = read_file(output_file)
@@ -99,6 +106,7 @@ def test_write_orbitize_input():
         # Remove temporary file
         os.remove(output_file)
 
+
 @deprecation.fail_if_not_removed
 def test_write_orbitize_input_2():
     """
@@ -114,7 +122,7 @@ def test_write_orbitize_input_2():
         os.remove(output_file)
     try:  # Catch these tests so that we remove temporary file
         # Test that we were able to write the table
-        write_orbitize_input(test_table,output_file)
+        write_orbitize_input(test_table, output_file)
         assert os.path.isfile(output_file)
         # Test that we can read the table and check if it's correct
         test_table_2 = read_orbitize_input(output_file)
@@ -123,8 +131,29 @@ def test_write_orbitize_input_2():
         # Remove temporary file
         os.remove(output_file)
 
+def test_cov_input():
+    """
+    Test including radec and seppa covariances/correlations.
+    """
+    testdir = orbitize.DATADIR
+    # Check that main test input is read in with correct values
+    input_file = os.path.join(testdir, 'test_val_cov.csv')
+    input_data = read_file(input_file)
+    _compare_table(input_data)
+    print(input_data)
+    # Check the covariance column
+    quant12_corr_truth = [0.25, np.nan, -0.5, np.nan]
+    assert 'quant12_corr' in input_data.columns
+    for row, truth in zip(input_data, quant12_corr_truth):
+        meas = row['quant12_corr']
+        if np.isnan(truth):
+            assert np.isnan(meas)
+        else:
+            assert truth == pytest.approx(meas)
+
 if __name__ == "__main__":
     test_read_file()
     test_read_formatted_file()
     test_write_orbitize_input()
     test_write_orbitize_input_2()
+    test_cov_input()
